@@ -25,7 +25,7 @@
    - indices are provided from 1 to n, i.e. not as C indices (from 0 to n-1)
 */
 
-void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, gentime *gen, param *par, int step, gsl_rng *rng, bool quiet){
+void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, spatial_dist *spainfo, gentime *gen, param *par, int step, gsl_rng *rng, bool quiet){
     int i;
     double like, prior;
 
@@ -34,7 +34,7 @@ void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, gentime *gen, param
     fprintf(file,"\n%d", step);
 
     /* posterior, likelihood, prior */
-    like = loglikelihood_all(dat, dnainfo, gen, par, rng);
+    like = loglikelihood_all(dat, dnainfo, spainfo, gen, par, rng);
     prior = logprior_all(par);
     fprintf(file,"\t%.15f", like + prior);
     fprintf(file,"\t%.15f", like);
@@ -45,7 +45,8 @@ void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, gentime *gen, param
     fprintf(file,"\t%.15f", par->mu1 * par->gamma);
     fprintf(file,"\t%.15f", par->gamma);
     fprintf(file,"\t%.15f", par->pi);
-    /* fprintf(file,"\t%.15f", par->phi); */
+    fprintf(file,"\t%.15f", par->spa_param1);
+    fprintf(file,"\t%.15f", par->spa_param2);
     for(i=0;i<par->Tinf->length;i++){
 	fprintf(file, "\t%d", vec_int_i(par->Tinf, i));
     }
@@ -66,7 +67,8 @@ void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, gentime *gen, param
 	Rprintf("\t%.15f", par->mu1 * par->gamma);
 	Rprintf("\t%.15f", par->gamma);
 	Rprintf("\t%.15f", par->pi);
-	/* printf("\t%.15f", par->phi); */
+	Rprintf("\t%.15f", par->spa_param1);
+	Rprintf("\t%.15f", par->spa_param2);
 	for(i=0;i<par->Tinf->length;i++){
 	    Rprintf("\t%d", vec_int_i(par->Tinf, i));
 	}
@@ -84,27 +86,32 @@ void fprint_chains(FILE *file, data *dat, dna_dist *dnainfo, gentime *gen, param
 
 
 
-/* print mcmc parameter (e.g. acceptance/rejection) to file 
+/* print mcmc parameter (e.g. acceptance/rejection) to file
    order is as follows:
-   step | global prop accept | accept_mu1 | sigma_mu1 | sigma_gamma
+   step | global prop accept | accept_mu1 | sigma_mu1 | sigma_gamma | sigma_spa1 | sigma_spa2
 */
 void fprint_mcmc_param(FILE *file, mcmc_param *mcmcPar, int step){
     double temp=0.0;
     /* OUTPUT TO FILE */
-    fprintf(file,"\n%d\t", step);
+    fprintf(file,"\n%d", step);
     temp = (double) mcmcPar->n_accept_mu1 / (double) (mcmcPar->n_accept_mu1 + mcmcPar->n_reject_mu1);
     fprintf(file,"\t%.5f", temp);
     temp = (double) mcmcPar->n_accept_gamma / (double) (mcmcPar->n_accept_gamma + mcmcPar->n_reject_gamma);
     fprintf(file,"\t%.5f", temp);
     temp = (double) mcmcPar->n_accept_pi / (double) (mcmcPar->n_accept_pi + mcmcPar->n_reject_pi);
     fprintf(file,"\t%.5f", temp);
-    /* temp = (double) mcmcPar->n_accept_phi / (double) (mcmcPar->n_accept_phi + mcmcPar->n_reject_phi); */
-    /* fprintf(file,"\t%.5f", temp); */
     temp = (double) mcmcPar->n_accept_Tinf / (double) (mcmcPar->n_accept_Tinf + mcmcPar->n_reject_Tinf);
     fprintf(file,"\t%.5f", temp);
+    temp = (double) mcmcPar->n_accept_spa1 / (double) (mcmcPar->n_accept_spa1 + mcmcPar->n_reject_spa1);
+    fprintf(file,"\t%.5f", temp);
+    temp = (double) mcmcPar->n_accept_spa2 / (double) (mcmcPar->n_accept_spa2 + mcmcPar->n_reject_spa2);
+    fprintf(file,"\t%.5f", temp);
+  
     fprintf(file,"\t%.15f", mcmcPar->sigma_mu1);
     fprintf(file,"\t%.15f", mcmcPar->sigma_gamma);
     fprintf(file,"\t%.15f", mcmcPar->sigma_pi);
+    fprintf(file,"\t%.15f", mcmcPar->sigma_spa1);
+    fprintf(file,"\t%.15f", mcmcPar->sigma_spa2);
     /* fprintf(file,"\t%.15f", mcmcPar->sigma_phi); */
     fprintf(file,"\t%d", mcmcPar->n_like_zero);
 }
@@ -126,6 +133,8 @@ void fprint_mcmc_param(FILE *file, mcmc_param *mcmcPar, int step){
 /*
    AIM: get ~40% acceptance for univariate param, ~20% for multivariate param
 */
+
+/* tune moves for mu1 */
 void tune_mu1(mcmc_param * in, gsl_rng *rng){
     /* get acceptance proportion */
     double paccept = (double) in->n_accept_mu1 / (double) (in->n_accept_mu1 + in->n_reject_mu1);
@@ -153,6 +162,7 @@ void tune_mu1(mcmc_param * in, gsl_rng *rng){
 
 
 
+/* tune moves for gamma */
 void tune_gamma(mcmc_param * in, gsl_rng *rng){
     /* get acceptance proportion */
     double paccept = (double) in->n_accept_gamma / (double) (in->n_accept_gamma + in->n_reject_gamma);
@@ -180,6 +190,7 @@ void tune_gamma(mcmc_param * in, gsl_rng *rng){
 
 
 
+/* tune moves for pi */
 void tune_pi(mcmc_param * in, gsl_rng *rng){
     /* get acceptance proportion */
     double paccept = (double) in->n_accept_pi / (double) (in->n_accept_pi + in->n_reject_pi);
@@ -208,6 +219,62 @@ void tune_pi(mcmc_param * in, gsl_rng *rng){
 
 
 
+/* tune moves for spa1 */
+void tune_spa1(mcmc_param * in, gsl_rng *rng){
+    /* get acceptance proportion */
+    double paccept = (double) in->n_accept_spa1 / (double) (in->n_accept_spa1 + in->n_reject_spa1);
+
+    /* acceptable zone: 25-50% acceptance */
+    if(paccept<0.25) {
+	in->sigma_spa1 /= 1.5;
+	in->n_accept_spa1 = 0;
+	in->n_reject_spa1 = 0;
+    } else if (paccept>0.50) {
+	in->sigma_spa1 *= 1.5;
+	in->n_accept_spa1 = 0;
+	in->n_reject_spa1 = 0;
+	/* do not allow sigma to be > 1 (for lognormal not to go crazy) */
+	if(in->sigma_spa1>1.0){
+	    in->sigma_spa1 = 1.0;
+	    in->tune_spa1 = FALSE;
+	}
+    } else {
+	in->tune_spa1 = FALSE;
+    }
+}
+
+
+
+
+
+/* tune moves for spa2 */
+void tune_spa2(mcmc_param * in, gsl_rng *rng){
+    /* get acceptance proportion */
+    double paccept = (double) in->n_accept_spa2 / (double) (in->n_accept_spa2 + in->n_reject_spa2);
+
+    /* acceptable zone: 25-50% acceptance */
+    if(paccept<0.25) {
+	in->sigma_spa2 /= 1.5;
+	in->n_accept_spa2 = 0;
+	in->n_reject_spa2 = 0;
+    } else if (paccept>0.50) {
+	in->sigma_spa2 *= 1.5;
+	in->n_accept_spa2 = 0;
+	in->n_reject_spa2 = 0;
+	/* do not allow sigma to be > 1 (for lognormal not to go crazy) */
+	if(in->sigma_spa2>1.0){
+	    in->sigma_spa2 = 1.0;
+	    in->tune_spa2 = FALSE;
+	}
+    } else {
+	in->tune_spa2 = FALSE;
+    }
+}
+
+
+
+
+
 /* void tune_phi(mcmc_param * in, gsl_rng *rng){ */
 /*     /\* get acceptance proportion *\/ */
 /*     double paccept = (double) in->n_accept_phi / (double) (in->n_accept_phi + in->n_reject_phi); */
@@ -258,10 +325,10 @@ void tune_pi(mcmc_param * in, gsl_rng *rng){
 
 /* PRELIM MCMC FOR FINDING OUTLIERS */
 void mcmc_find_import(vec_int *areOutliers, int outEvery, int tuneEvery, bool quiet, param *par, 
-		      data *dat, dna_dist *dnainfo, gentime *gen, mcmc_param *mcmcPar, gsl_rng *rng){
+		      data *dat, dna_dist *dnainfo, spatial_dist *spainfo, gentime *gen, mcmc_param *mcmcPar, gsl_rng *rng){
 
-    int i, j, nbTermsLike = 0;
-    double medLogLike = 0.0;
+  int i, j, nbTermsLike = 0, nbCasesWithInfluence = 0;
+    double meanInfluence = 0.0;
 
     /* OUTPUT TO SCREEN - HEADER */
     if(!quiet){
@@ -287,20 +354,35 @@ void mcmc_find_import(vec_int *areOutliers, int outEvery, int tuneEvery, bool qu
     mcmc_param *localMcmcPar = alloc_mcmc_param(dat->n);
     copy_mcmc_param(mcmcPar, localMcmcPar);
 
-    /* CREATE TEMPORARY VECTOR STORING INDIVIDUAL LIKELIHOODS */
-    vec_double *indivLogLike = alloc_vec_double(dat->n);
+    /* CREATE TEMPORARY VECTOR STORING "GLOBAL INFLUENCE" OF EACH INDIVIDUALS */
+    /* let LL be the total log-likelihood, LL[-i] the same without observation i */
+    /* and LL(i) the contrib of 'i' to the total log-likelihood */
+    /* Then: */
+    /* GI_i = LL[-i] - LL = -LL(i) */
+    /* See Hens et al. (2012) AJE, Doi: 10.1093/aje/kws006 */
+
+    vec_double *indivInfluence = alloc_vec_double(dat->n);
 
     /* RUN MCMC */
     for(i=2;i<=localMcmcPar->find_import_at;i++){
 	/* printf("i: %d ",i);fflush(stdout); */
-	/* COLLECT INFORMATION ABOUT INDIVIDUAL LIKELIHOODS */
+	/* COLLECT INFORMATION ABOUT ALL GI_i */
 	if(i>=localMcmcPar->burnin && i % outEvery == 0){
-	    /* printf("\ni=%d - computing individual likelihoods\n",i);fflush(stdout); */
 	    for(j=0;j<dat->n;j++){
-		indivLogLike->values[j] += loglikelihood_gen_i(j,dat, dnainfo, localPar, rng);
+		indivInfluence->values[j] -= loglikelihood_gen_i(j,dat, dnainfo, localPar, rng);
 	    }
-	    /* printf("\nlikelihood vector:\n");fflush(stdout); */
-	    /* print_vec_double(indivLogLike); */
+	    /* DEBUGGING */
+	    /* printf("\ninfluence values:\n");fflush(stdout); */
+	    /* print_vec_double(indivInfluence); */
+
+	    /* printf("\nancestries:\n");fflush(stdout); */
+	    /* print_vec_int(par->alpha); */
+
+	    /* printf("\nmost recent sequenced ancestors:\n");fflush(stdout); */
+	    /* for(j=0;j<dat->n;j++){ */
+	    /*   printf("\nancestor of %d: %d",j,find_sequenced_ancestor(j, dat, dnainfo, par)); */
+	    /*   fflush(stdout); */
+	    /* } */
 	    nbTermsLike++;
 	}
 
@@ -311,7 +393,7 @@ void mcmc_find_import(vec_int *areOutliers, int outEvery, int tuneEvery, bool qu
 	    tune_pi(localMcmcPar,rng);
 	    /* tune_phi(localMcmcPar,rng); */
 	    /* localMcmcPar->tune_all = localMcmcPar->tune_mu1 || localMcmcPar->tune_gamma || localMcmcPar->tune_pi || localMcmcPar->tune_phi; */
-	    localMcmcPar->tune_all = localMcmcPar->tune_mu1 || localMcmcPar->tune_gamma || localMcmcPar->tune_pi;
+	    localMcmcPar->tune_all = localMcmcPar->tune_mu1 || localMcmcPar->tune_gamma || localMcmcPar->tune_pi || localMcmcPar->tune_spa1 || localMcmcPar->tune_spa2;
 	}
 
 	/* MOVEMENTS */
@@ -327,56 +409,75 @@ void mcmc_find_import(vec_int *areOutliers, int outEvery, int tuneEvery, bool qu
 	/* move pi */
 	if(localMcmcPar->move_pi) move_pi(localPar, tempPar, dat, localMcmcPar, rng);
 
-	/* /\* move phi *\/ */
-	/* if(localMcmcPar->move_phi) move_phi(localPar, tempPar, dat, localMcmcPar, rng); */
+	/* move spa1 */
+	if(localMcmcPar->move_spa1) move_spa1(localPar, tempPar, dat, spainfo, localMcmcPar, rng);
+
+	/* move spa2 */
+	if(localMcmcPar->move_spa2) move_spa2(localPar, tempPar, dat, spainfo, localMcmcPar, rng);
+
+	/* move Tinf, kappa_i and alpha_i alltogether */
+	move_Tinf_alpha_kappa(localPar, tempPar, dat, dnainfo, spainfo, gen, localMcmcPar, rng);
 
 	/* move Tinf */
-	if(localMcmcPar->move_Tinf) move_Tinf(localPar, tempPar, dat, dnainfo, gen, localMcmcPar, rng);
+	if(localMcmcPar->move_Tinf) move_Tinf(localPar, tempPar, dat, dnainfo, spainfo, gen, localMcmcPar, rng);
 
-	/* /\* move alpha_i*\/ */
-	/* move_alpha(localPar, tempPar, dat, dnainfo, gen, localMcmcPar, rng); */
+	/* swap ancestries */
+	swap_ancestries(localPar, tempPar, dat, dnainfo, spainfo, gen, localMcmcPar, rng);
 
-	/* /\* move kappa_i*\/ */
-	/* move_kappa(localPar, tempPar, dat, dnainfo, gen, localMcmcPar, rng); */
-
-	/* move alpha_i and kappa_i*/
-	move_alpha_kappa(localPar, tempPar, dat, dnainfo, gen, localMcmcPar, rng);
+	/* /\* move alpha_i and kappa_i*\/ */
+	/* move_alpha_kappa(localPar, tempPar, dat, dnainfo, spainfo, gen, localMcmcPar, rng); */
 
     } /* end of MCMC */
 
 
     /* FIND IMPORTED CASES */
-    /* compute individual average log-like */
+    /* compute average GI_i for each individual */
+    /* also compute mean influence across sequenced individuals */
+    meanInfluence = 0.0;
+    nbCasesWithInfluence = 0;
     for(j=0;j<dat->n;j++){
-	indivLogLike->values[j] = vec_double_i(indivLogLike,j)/((double) nbTermsLike);
-    }
+      /* influence for individuals */
+      indivInfluence->values[j] = vec_double_i(indivInfluence,j)/((double) nbTermsLike);
 
-    /* compute general average log-like */
-    medLogLike = mean_vec_double(indivLogLike);
-    /* printf("\nAverage loglike: %f\n", medLogLike);fflush(stdout); */
-    /* printf("\nIndividual loglike:\n");fflush(stdout); */
-    /* print_vec_double(indivLogLike); */
-
-    /* browse each likelihood, define outliers */
-    /* printf("\n\nLooking for outliers...\n"); */
-    for(j=0;j<dat->n;j++){
-	/* outliers = likelihood xxx times lower than the mean */
-	/* ('xxx' defined in par) */
-	/* printf("\nIndiv %d: loglike difference= %.5f", j+1, medLogLike - vec_double_i(indivLogLike,j));fflush(stdout); */
-	if((medLogLike - vec_double_i(indivLogLike,j)) > log(par->outlier_threshold)){
-	    areOutliers->values[j] = 1;
-	    Rprintf("\nIndividual %d identified as imported case\n",j+1);
-	} else {
-	    areOutliers->values[j] = 0;
+      /* average influence across individuals */
+      /* (only cases with a genetic sequence are taken into account) */
+  	if(vec_int_i(dat->idxCasesInDna, j)>=0) {
+	  meanInfluence += indivInfluence->values[j];
+	  nbCasesWithInfluence++;
 	}
-    } /* end setting outliers */
+    }
+    meanInfluence = meanInfluence/nbCasesWithInfluence;
 
+    /* meanInfluence = mean_vec_double(indivInfluence); */
+    Rprintf("\nAverage influence: %f\n", meanInfluence);
+    Rprintf("\nIndividual influences:\n");
+    print_vec_double(indivInfluence);
+    Rprintf("\nThreshold (x%d) for outlier classification: influence > %.5f\n", (int) par->outlier_threshold, par->outlier_threshold*meanInfluence);
+
+    /* browse global influences and define outliers */
+    /* (only if at least 5 cases have a computable influence) */
+    /* printf("\n\nLooking for outliers...\n"); */
+    if(nbCasesWithInfluence>4){
+      for(j=0;j<dat->n;j++){
+	/* outliers = GI_i xxx times larger than the mean */
+	/* ('xxx' defined in par) */
+	/* if((medLogLike - vec_double_i(indivInfluence,j)) > log(par->outlier_threshold)){ */
+	if(vec_double_i(indivInfluence,j) > (par->outlier_threshold * meanInfluence)){
+	  areOutliers->values[j] = 1;
+	  Rprintf("\nIndividual %d identified as imported case\n",j+1);
+	} else {
+	  areOutliers->values[j] = 0;
+	}
+      } /* end setting outliers */
+    } else {
+      Rprintf("\nLess than 5 cases have a genetic sequence - aborting outlier detection");
+    }
 
     /* FREE TEMPORARY PARAMETERS */
     free_param(localPar);
     free_param(tempPar);
     free_mcmc_param(localMcmcPar);
-    free_vec_double(indivLogLike);
+    free_vec_double(indivInfluence);
 } /* end mcmc_find_import */
 
 
@@ -392,7 +493,7 @@ void mcmc_find_import(vec_int *areOutliers, int outEvery, int tuneEvery, bool qu
    ==============
 */
 void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256], int tuneEvery, 
-	  bool quiet, param *par, data *dat, dna_dist *dnainfo, gentime *gen, mcmc_param *mcmcPar, gsl_rng *rng){
+	  bool quiet, param *par, data *dat, dna_dist *dnainfo, spatial_dist *spainfo, gentime *gen, mcmc_param *mcmcPar, gsl_rng *rng){
 
     int i;
     vec_int *areOutliers = alloc_vec_int(dat->n);
@@ -413,8 +514,7 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 
 
     /* OUTPUT TO OUTFILE - HEADER */
-    /* fprintf(file, "step\tpost\tlike\tprior\tmu1\tmu2\tgamma\tpi\tphi"); */
-    fprintf(file, "step\tpost\tlike\tprior\tmu1\tmu2\tgamma\tpi");
+    fprintf(file, "step\tpost\tlike\tprior\tmu1\tmu2\tgamma\tpi\tspa1\tspa2");
     for(i=0;i<dat->n;i++){
 	fprintf(file, "\tTinf_%d", i+1);
     }
@@ -426,15 +526,13 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
     }
 
     /* OUTPUT TO MCMCOUTFILE - HEADER */
-    /* fprintf(mcmcFile, "step\tp_accept_mu1\tp_accept_gamma\tp_accept_pi\tp_accept_phi\tp_accept_Tinf"); */
-    /* fprintf(mcmcFile, "\tsigma_mu1\tsigma_gamma\tsigma_pi\tsigma_phi\tn_like_zero"); */
-    fprintf(mcmcFile, "step\tp_accept_mu1\tp_accept_gamma\tp_accept_pi\tp_accept_Tinf");
-    fprintf(mcmcFile, "\tsigma_mu1\tsigma_gamma\tsigma_pi\tn_like_zero");
+    fprintf(mcmcFile, "step\tp_accept_mu1\tp_accept_gamma\tp_accept_pi\tp_accept_Tinf\tp_accept_spa1\tp_accept_spa2");
+    fprintf(mcmcFile, "\tsigma_mu1\tsigma_gamma\tsigma_pi\tsigma_spa1\tsigma_spa2\tn_like_zero");
 
 
     /* OUTPUT TO SCREEN - HEADER */
     if(!quiet){
-	Rprintf("step\tpost\tlike\tprior\tmu1\tmu2\tgamma\tpi");
+	Rprintf("step\tpost\tlike\tprior\tmu1\tmu2\tgamma\tpi\tspa1\tspa2");
 	for(i=0;i<dat->n;i++){
 	    Rprintf("\tTinf_%d", i+1);
 	}
@@ -446,7 +544,7 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 	}
     }
 
-    fprint_chains(file, dat, dnainfo, gen, par, 1, rng, quiet);
+    fprint_chains(file, dat, dnainfo, spainfo, gen, par, 1, rng, quiet);
     fprint_mcmc_param(mcmcFile, mcmcPar, 1);
 
     mcmcPar->step_notune = nIter;
@@ -454,7 +552,7 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 
     /* PRELIM STEP - FINDING OUTLIERS */
     if(mcmcPar->find_import){
-	mcmc_find_import(areOutliers, outEvery, tuneEvery, quiet, par, dat, dnainfo, gen, mcmcPar, rng);
+	mcmc_find_import(areOutliers, outEvery, tuneEvery, quiet, par, dat, dnainfo, spainfo, gen, mcmcPar, rng);
 
 	/* RESTORE INITIAL TUNING SETTINGS AND PARAM */
 	/* mcmcPar->tune_all = TRUE; */
@@ -485,7 +583,7 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 
 	/* OUTPUT TO FILES */
 	if(i % outEvery == 0){
-	    fprint_chains(file, dat, dnainfo, gen, par, i, rng, quiet);
+	    fprint_chains(file, dat, dnainfo, spainfo, gen, par, i, rng, quiet);
 	    fprint_mcmc_param(mcmcFile, mcmcPar, i);
 	}
 
@@ -496,7 +594,7 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 	    tune_pi(mcmcPar,rng);
 	    /* tune_phi(mcmcPar,rng); */
 	    /* mcmcPar->tune_all = mcmcPar->tune_mu1 || mcmcPar->tune_gamma || mcmcPar->tune_pi || mcmcPar->tune_phi; */
-	    mcmcPar->tune_all = mcmcPar->tune_mu1 || mcmcPar->tune_gamma || mcmcPar->tune_pi;
+	    mcmcPar->tune_all = mcmcPar->tune_mu1 || mcmcPar->tune_gamma || mcmcPar->tune_pi || mcmcPar->tune_spa1 || mcmcPar->tune_spa2;
 	    if(!mcmcPar->tune_all) {
 		mcmcPar->step_notune = i;
 		/* printf("\nStopped tuning at chain %d\n",i);fflush(stdout); */
@@ -511,33 +609,34 @@ void mcmc(int nIter, int outEvery, char outputFile[256], char mcmcOutputFile[256
 	/* check_loglikelihood_all(dat, dnainfo, gen, par); */
 
 	/* MOVEMENTS */
-	if(mcmcPar->move_mut){/* move mu1 */
-	    move_mu1(par, tempPar, dat, dnainfo, mcmcPar, rng);
+	if(mcmcPar->move_mut){
+	    /* move mu1 */
+	    if(par->mut_model==1 || par->mut_model==2) move_mu1(par, tempPar, dat, dnainfo, mcmcPar, rng);
 
 	    /* move gamma */
-	    move_gamma(par, tempPar, dat, dnainfo, mcmcPar, rng);
+	    if(par->mut_model==2) move_gamma(par, tempPar, dat, dnainfo, mcmcPar, rng);
 	}
 
 	/* move pi */
 	if(mcmcPar->move_pi) move_pi(par, tempPar, dat, mcmcPar, rng);
 
-	/* /\* move phi *\/ */
-	/* if(mcmcPar->move_phi) move_phi(par, tempPar, dat, mcmcPar, rng); */
+	/* move spa1 */
+	if(mcmcPar->move_spa1) move_spa1(par, tempPar, dat, spainfo, mcmcPar, rng);
+
+	/* move spa2 */
+	if(mcmcPar->move_spa2) move_spa2(par, tempPar, dat, spainfo, mcmcPar, rng);
+
+	/* move Tinf, kappa_i and alpha_i alltogether */
+	move_Tinf_alpha_kappa(par, tempPar, dat, dnainfo, spainfo, gen, mcmcPar, rng);
 
 	/* move Tinf */
-	/* printf("\nTinf:"); */
-	/* print_vec_int(par->Tinf); */
-	/* fflush(stdout); */
-	if(mcmcPar->move_Tinf) move_Tinf(par, tempPar, dat, dnainfo, gen, mcmcPar, rng);
+	if(mcmcPar->move_Tinf) move_Tinf(par, tempPar, dat, dnainfo, spainfo, gen, mcmcPar, rng);
 
-	/* /\* move alpha_i*\/ */
-	/* move_alpha(par, tempPar, dat, dnainfo, gen, mcmcPar, rng); */
+	/* swap ancestries */
+	swap_ancestries(par, tempPar, dat, dnainfo, spainfo, gen, mcmcPar, rng);
 
-	/* /\* move kappa_i*\/ */
-	/* move_kappa(par, tempPar, dat, dnainfo, gen, mcmcPar, rng); */
-
-	/* move alpha_i and kappa_i*/
-	move_alpha_kappa(par, tempPar, dat, dnainfo, gen, mcmcPar, rng);
+	/* /\* move alpha_i and kappa_i*\/ */
+	/* move_alpha_kappa(par, tempPar, dat, dnainfo, spainfo, gen, mcmcPar, rng); */
 
     } /* end of mcmc */
 
